@@ -1,4 +1,4 @@
-/* $Id: network_server.cpp 26094 2013-11-24 19:57:23Z rubidium $ */
+/* $Id: network_server.cpp 26625 2014-06-02 18:18:35Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -2171,4 +2171,37 @@ void NetworkPrintClients()
 	}
 }
 
+/**
+ * Perform all the server specific administration of a new company.
+ * @param c  The newly created company; can't be NULL.
+ * @param ci The client information of the client that made the company; can be NULL.
+ */
+void NetworkServerNewCompany(const Company *c, NetworkClientInfo *ci)
+{
+	assert(c != NULL);
+
+	if (!_network_server) return;
+
+	_network_company_states[c->index].months_empty = 0;
+	_network_company_states[c->index].password[0] = '\0';
+	NetworkServerUpdateCompanyPassworded(c->index, false);
+
+	if (ci != NULL) {
+		/* ci is NULL when replaying, or for AIs. In neither case there is a client. */
+		ci->client_playas = c->index;
+		NetworkUpdateClientInfo(ci->client_id);
+		NetworkSendCommand(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, c->index);
+	}
+
+	/* Announce new company on network. */
+	NetworkAdminCompanyInfo(c, true);
+
+	if (ci != NULL) {
+		/* ci is NULL when replaying, or for AIs. In neither case there is a client.
+		   We need to send Admin port update here so that they first know about the new company
+		   and then learn about a possibly joining client (see FS#6025) */
+		NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, c->index + 1);
+	}
+}
+
 #endif /* ENABLE_NETWORK */
